@@ -170,7 +170,7 @@ particle_smoother.nlg_ssm <- function(object, nsim,
   max_iter = 100, conv_tol = 1e-8, iekf_iter = 0, ...) {
   
   smoothing_method <- match.arg(smoothing_method, c("fs", "fbs"))
-  filter_type <- match.arg(filter_type, c("bsf", "psi", "apf", "ekf", "psi_df"))
+  filter_type <- match.arg(filter_type, c("bsf", "psi", "apf", "ekf"))
   if (smoothing_method == "fbs") {
     stop("FBS is not yet implemented.")
   }
@@ -197,13 +197,7 @@ particle_smoother.nlg_ssm <- function(object, nsim,
       object$theta, object$log_prior_pdf, object$known_params, 
       object$known_tv_params, object$n_states, object$n_etas, 
       as.integer(object$time_varying), nsim, 
-      seed),
-    psi_df = df_psi_smoother_nlg(t(object$y), object$Z, object$H, object$T, 
-      object$R, object$Z_gn, object$T_gn, object$a1, object$P1, 
-      object$theta, object$log_prior_pdf, object$known_params, 
-      object$known_tv_params, object$n_states, object$n_etas, 
-      as.integer(object$time_varying), nsim, seed,
-      max_iter, conv_tol, iekf_iter)
+      seed)
   )
   colnames(out$alphahat) <- colnames(out$Vt) <-
     colnames(out$Vt) <- object$state_names
@@ -213,3 +207,27 @@ particle_smoother.nlg_ssm <- function(object, nsim,
   out$alpha <- aperm(out$alpha, c(2, 1, 3))
   out
 }
+
+
+#' @rdname particle_smoother
+#' @method particle_smoother sde_ssm
+#' @param L Integer defining the discretization level.
+#' @export
+particle_smoother.sde_ssm <- function(object, nsim, L, 
+seed = sample(.Machine$integer.max, size = 1), ...) {
+  
+  if(L < 1) stop("Discretization level L must be larger than 0.")
+  out <-  bsf_smoother_sde(object$y, object$x0, object$positive, 
+    object$drift, object$diffusion, object$ddiffusion, 
+    object$prior_pdf, object$obs_pdf, object$theta, 
+    nsim, round(L), seed)
+  
+  colnames(out$alphahat) <- colnames(out$Vt) <-
+    colnames(out$Vt) <- object$state_names
+  out$alphahat <- ts(out$alphahat, start = start(object$y), frequency = frequency(object$y))
+  
+  rownames(out$alpha) <- object$state_names
+  out$alpha <- aperm(out$alpha, c(2, 1, 3))
+  out
+}
+
