@@ -22,7 +22,7 @@ print.mcmc_output <- function(x, ...) {
   theta <- mcmc(x$theta)
   alpha <- mcmc(matrix(x$alpha[nrow(x$alpha),,], ncol = ncol(x$alpha), byrow = TRUE, 
     dimnames = list(NULL, colnames(x$alpha))))
-  w <- x$counts * if (x$isc) x$weights else 1
+  w <- x$counts * if (x$mcmc_type %in% paste0("is", 1:3)) x$weights else 1
   
   
   cat("\nCall:\n", paste(deparse(x$call), sep = "\n", collapse = "\n"), 
@@ -39,8 +39,9 @@ print.mcmc_output <- function(x, ...) {
   se_theta_is <- weighted_se(theta, w)
   spec <- sapply(1:ncol(theta), function(i) spectrum0.ar((theta[, i] - mean_theta[i]) * w)$spec)
   se_theta_ar <- sqrt(spec / length(w)) / mean(w)
-  stats <- matrix(c(mean_theta, sd_theta, se_theta_is, se_theta_ar), ncol = 4, 
-    dimnames = list(colnames(x$theta), c("Mean", "SD", "SE-IS", "SE-AR")))
+  se_theta_total <- sqrt(se_theta_is^2 + se_theta_ar^2)
+  stats <- matrix(c(mean_theta, sd_theta, se_theta_is, se_theta_ar, se_theta_total), ncol = 5, 
+    dimnames = list(colnames(x$theta), c("Mean", "SD", "SE-IS", "SE-AR", "SE")))
   print(stats)
   
   cat("\nEffective sample sizes for theta:\n\n")
@@ -58,8 +59,9 @@ print.mcmc_output <- function(x, ...) {
   se_alpha_is <- weighted_se(alpha, w)
   spec <- sapply(1:ncol(alpha), function(i) spectrum0.ar((alpha[, i] - mean_alpha[i]) * w)$spec)
   se_alpha_ar <- sqrt(spec / length(w)) / mean(w)
-  stats <- matrix(c(mean_alpha, sd_alpha, se_alpha_is, se_alpha_ar), ncol = 4, 
-    dimnames = list(colnames(x$alpha), c("Mean", "SD", "SE-IS", "SE-AR")))
+  se_alpha_total <- sqrt(se_alpha_is^2 + se_alpha_ar^2)
+  stats <- matrix(c(mean_alpha, sd_alpha, se_alpha_is, se_alpha_ar, se_alpha_total), ncol = 5, 
+    dimnames = list(colnames(x$alpha), c("Mean", "SD", "SE-IS", "SE-AR", "SE")))
   print(stats)
   
   cat("\nEffective sample sizes for alpha:\n\n")
@@ -81,56 +83,64 @@ print.mcmc_output <- function(x, ...) {
 #' calls to \code{\link[coda]{spectrum0.ar}}.
 #' 
 #' @param object Output from \code{run_mcmc}
+#' @param return_se if \code{FALSE} (default), computation of standard 
+#' errors and effective sample sizes is omitted. 
+#' This saves time, as computing the spectral densities (by \code{coda}) can be slow for 
+#' large models.
 #' @param only_theta If \code{TRUE}, summaries are computed only for hyperparameters theta. 
 #' @param ... Ignored.
 #' @export
-summary.mcmc_output <- function(object, only_theta = FALSE, ...) {
+summary.mcmc_output <- function(object, return_se = FALSE, only_theta = FALSE, ...) {
+  
   
-  if (only_theta) {
-    summaries <- list(theta = NULL)
-  } else {
-    summaries <- list(theta = NULL, alpha_mean = NULL, alpha_sd = NULL, alpha_se_is = NULL, 
-      alpha_se_ar = NULL, alpha_ess_is = NULL, alpha_ess_ar = NULL)
-  }
   theta <- mcmc(object$theta)
-  w <- object$counts * if (object$isc) object$weights else 1
+  w <- object$counts * if (object$mcmc_type %in% paste0("is", 1:3)) object$weights else 1
   
   mean_theta <- weighted_mean(theta, w)
   sd_theta <- sqrt(diag(weighted_var(theta, w, method = "moment")))
-  se_theta_is <- weighted_se(theta, w)
-  spec <- sapply(1:ncol(theta), function(i) spectrum0.ar((theta[, i] - mean_theta[i]) * w)$spec)
-  se_theta_ar <- sqrt(spec / length(w)) / mean(w)
-  stats <- matrix(c(mean_theta, sd_theta, se_theta_is, se_theta_ar), ncol = 4, 
-    dimnames = list(colnames(object$theta), c("Mean", "SD", "SE-IS", "SE-AR")))
+  if(return_se) {
+    se_theta_is <- weighted_se(theta, w)
+    spec <- sapply(1:ncol(theta), function(i) spectrum0.ar((theta[, i] - mean_theta[i]) * w)$spec)
+    se_theta_ar <- sqrt(spec / length(w)) / mean(w)
+    se_theta_total <- sqrt(se_theta_is^2 + se_theta_ar^2)
+    ess_theta_is <- apply(theta, 2, function(z) ess(w, identity, z))
+    ess_theta_ar <- (sd_theta / se_theta_ar)^2
+    summary_theta <- matrix(c(
+      mean_theta, sd_theta, se_theta_is, se_theta_ar, se_theta_total,
+      ess_theta_is, ess_theta_ar), 
+      ncol = 7, 
+      dimnames = list(
+        colnames(object$theta), 
+        c("Mean", "SD", "SE-IS", "SE-AR", "SE", "ESS-IS", "ESS-AR")))
+  } else {
+    summary_theta <- matrix(c(mean_theta, sd_theta), ncol = 2, 
+      dimnames = list(colnames(object$theta), c("Mean", "SD")))
+  }
   
-  
-  ess_theta_is <- apply(theta, 2, function(z) ess(w, identity, z))
-  ess_theta_ar <- (sd_theta / se_theta_ar)^2
-  esss <- matrix(c(ess_theta_is, ess_theta_ar), ncol = 2, 
-    dimnames = list(colnames(object$theta), c("ESS-IS", "ESS-AR")))
-  
-  summaries$theta <- cbind(stats, esss)
   if (!only_theta) {
     m <- ncol(object$alpha)
     mean_alpha <- weighted_mean(object$alpha, w)
     sd_alpha <- weighted_var(object$alpha, w, method = "moment")
     sd_alpha <- if(m > 1) sqrt(t(apply(sd_alpha, 3, diag))) else matrix(sqrt(sd_alpha), ncol = 1)
-    se_alpha_is <- apply(object$alpha, 2, function(x) weighted_se(t(x), w))
-    spec <- matrix(NA, ncol(object$alpha), nrow(object$alpha))
-    for(j in 1:nrow(object$alpha)) {
-      spec[, j] <- sapply(1:ncol(object$alpha), function(i) spectrum0.ar((object$alpha[j, i, ] - mean_alpha[j, i]) * w)$spec)
+    if(return_se) {
+      se_alpha_is <- apply(object$alpha, 2, function(x) weighted_se(t(x), w))
+      spec <- matrix(NA, ncol(object$alpha), nrow(object$alpha))
+      for(j in 1:nrow(object$alpha)) {
+        spec[, j] <- sapply(1:ncol(object$alpha), function(i) spectrum0.ar((object$alpha[j, i, ] - mean_alpha[j, i]) * w)$spec)
+      }
+      se_alpha_ar <- sqrt(t(spec) / length(w)) / mean(w)
+      se_alpha_total <- sqrt(se_alpha_is^2 + se_alpha_ar^2)
+      alpha_ess_is <- apply(object$alpha, 2, function(x) apply(t(x), 2, function(z) ess(w, identity, z)))
+      alpha_ess_ar <- (sd_alpha / se_alpha_ar)^2
+      summary_alpha <- list(
+        "Mean" = mean_alpha, "SD" = sd_alpha, 
+        "SE-IS" = se_alpha_is, "SE-AR" = se_alpha_ar, 
+        "SE" = se_alpha_total, "ESS-IS" = alpha_ess_is, "ESS-AR" = alpha_ess_ar)
+    } else {
+      summary_alpha <- list("Mean" = mean_alpha, "SD" = sd_alpha)
     }
-    se_alpha_ar <- sqrt(t(spec) / length(w)) / mean(w)
-    
-    summaries$alpha_mean <- mean_alpha
-    summaries$alpha_sd <- sd_alpha
-    summaries$alpha_se_is <- se_alpha_is
-    summaries$alpha_se_ar <- se_alpha_ar
-    
-    summaries$alpha_ess_is <- apply(object$alpha, 2, function(x) apply(t(x), 2, function(z) ess(w, identity, z)))
-    summaries$alpha_ess_ar <- (sd_alpha / se_alpha_ar)^2
-  }
-  summaries
+    return(list(theta = summary_theta, states = summary_alpha))
+  } else summary_theta
 }
 
 #' Expand the Jump Chain representation
