@@ -4,16 +4,27 @@
 #include "summary.h"
 
 // [[Rcpp::export]]
-Rcpp::List ekpf(const arma::mat& y, SEXP Z_fn_, SEXP H_fn_, 
-  SEXP T_fn_, SEXP R_fn_, SEXP Z_gn_, SEXP T_gn_, SEXP a1_fn_, SEXP P1_fn_, 
-  const arma::vec& theta, SEXP log_prior_pdf_, const arma::vec& known_params, 
+Rcpp::List ekpf(const arma::mat& y, SEXP Z, SEXP H, 
+  SEXP T, SEXP R, SEXP Zg, SEXP Tg, SEXP a1, SEXP P1, 
+  const arma::vec& theta, SEXP log_prior_pdf, const arma::vec& known_params, 
   const arma::mat& known_tv_params, const unsigned int n_states, 
   const unsigned int n_etas,  const arma::uvec& time_varying,
   const unsigned int nsim_states, 
   const unsigned int seed) {
   
-  nlg_ssm model(y, Z_fn_, H_fn_, T_fn_, R_fn_, Z_gn_, T_gn_, a1_fn_, P1_fn_, 
-    theta, log_prior_pdf_, known_params, known_tv_params, n_states, n_etas,
+  
+  Rcpp::XPtr<nvec_fnPtr> xpfun_Z(Z);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_H(H);
+  Rcpp::XPtr<nvec_fnPtr> xpfun_T(T);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_R(R);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_Zg(Zg);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_Tg(Tg);
+  Rcpp::XPtr<a1_fnPtr> xpfun_a1(a1);
+  Rcpp::XPtr<P1_fnPtr> xpfun_P1(P1);
+  Rcpp::XPtr<prior_fnPtr> xpfun_prior(log_prior_pdf);
+  
+  nlg_ssm model(y, *xpfun_Z, *xpfun_H, *xpfun_T, *xpfun_R, *xpfun_Zg, *xpfun_Tg, 
+    *xpfun_a1, *xpfun_P1,  theta, *xpfun_prior, known_params, known_tv_params, n_states, n_etas,
     time_varying, seed);
   
   unsigned int m = model.m;
@@ -42,17 +53,26 @@ Rcpp::List ekpf(const arma::mat& y, SEXP Z_fn_, SEXP H_fn_,
 }
 
 // [[Rcpp::export]]
-Rcpp::List ekpf_smoother(const arma::mat& y, SEXP Z_fn_, SEXP H_fn_, 
-  SEXP T_fn_, SEXP R_fn_, SEXP Z_gn_, SEXP T_gn_, SEXP a1_fn_, SEXP P1_fn_, 
-  const arma::vec& theta, SEXP log_prior_pdf_, const arma::vec& known_params, 
+Rcpp::List ekpf_smoother(const arma::mat& y, SEXP Z, SEXP H, 
+  SEXP T, SEXP R, SEXP Zg, SEXP Tg, SEXP a1, SEXP P1, 
+  const arma::vec& theta, SEXP log_prior_pdf, const arma::vec& known_params, 
   const arma::mat& known_tv_params, const unsigned int n_states, 
   const unsigned int n_etas,  const arma::uvec& time_varying,
   const unsigned int nsim_states, 
   const unsigned int seed) {
   
+  Rcpp::XPtr<nvec_fnPtr> xpfun_Z(Z);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_H(H);
+  Rcpp::XPtr<nvec_fnPtr> xpfun_T(T);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_R(R);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_Zg(Zg);
+  Rcpp::XPtr<nmat_fnPtr> xpfun_Tg(Tg);
+  Rcpp::XPtr<a1_fnPtr> xpfun_a1(a1);
+  Rcpp::XPtr<P1_fnPtr> xpfun_P1(P1);
+  Rcpp::XPtr<prior_fnPtr> xpfun_prior(log_prior_pdf);
   
-  nlg_ssm model(y, Z_fn_, H_fn_, T_fn_, R_fn_, Z_gn_, T_gn_, a1_fn_, P1_fn_, 
-    theta, log_prior_pdf_, known_params, known_tv_params, n_states, n_etas,
+  nlg_ssm model(y, *xpfun_Z, *xpfun_H, *xpfun_T, *xpfun_R, *xpfun_Zg, *xpfun_Tg, 
+    *xpfun_a1, *xpfun_P1,  theta, *xpfun_prior, known_params, known_tv_params, n_states, n_etas,
     time_varying, seed);
   
   unsigned int m = model.m;
