@@ -1,21 +1,22 @@
 #include "mgg_ssm.h"
+#include "psd_chol.h"
 
 // General constructor of mgg_ssm object from Rcpp::List
 // with parameter indices
-mgg_ssm::mgg_ssm(const Rcpp::List& model, const unsigned int seed, 
-  const arma::uvec& Z_ind, const arma::uvec& H_ind, 
+mgg_ssm::mgg_ssm(const Rcpp::List& model, const unsigned int seed,
+  const arma::uvec& Z_ind, const arma::uvec& H_ind,
   const arma::uvec& T_ind, const arma::uvec& R_ind) :
   y((Rcpp::as<arma::mat>(model["y"])).t()), Z(Rcpp::as<arma::cube>(model["Z"])),
-  H(Rcpp::as<arma::cube>(model["H"])), T(Rcpp::as<arma::cube>(model["T"])), 
-  R(Rcpp::as<arma::cube>(model["R"])), a1(Rcpp::as<arma::vec>(model["a1"])), 
-  P1(Rcpp::as<arma::mat>(model["P1"])), xreg(Rcpp::as<arma::cube>(model["xreg"])), 
-  beta(Rcpp::as<arma::mat>(model["coefs"])), 
+  H(Rcpp::as<arma::cube>(model["H"])), T(Rcpp::as<arma::cube>(model["T"])),
+  R(Rcpp::as<arma::cube>(model["R"])), a1(Rcpp::as<arma::vec>(model["a1"])),
+  P1(Rcpp::as<arma::mat>(model["P1"])), xreg(Rcpp::as<arma::cube>(model["xreg"])),
+  beta(Rcpp::as<arma::mat>(model["coefs"])),
   D(Rcpp::as<arma::mat>(model["obs_intercept"])),
-  C(Rcpp::as<arma::mat>(model["state_intercept"])), 
+  C(Rcpp::as<arma::mat>(model["state_intercept"])),
   Ztv(Z.n_slices > 1), Htv(H.n_slices > 1), Ttv(T.n_slices > 1), Rtv(R.n_slices > 1),
   Dtv(D.n_cols > 1), Ctv(C.n_cols > 1), n(y.n_cols), m(a1.n_elem), k(R.n_cols),
-  p(y.n_rows), HH(arma::cube(p, p, Htv * (n - 1) + 1)), 
-  RR(arma::cube(m, m, Rtv * (n - 1) + 1)), 
+  p(y.n_rows), HH(arma::cube(p, p, Htv * (n - 1) + 1)),
+  RR(arma::cube(m, m, Rtv * (n - 1) + 1)),
   xbeta(arma::mat(n, p, arma::fill::zeros)), engine(seed), zero_tol(1e-8),
   Z_ind(Z_ind), H_ind(H_ind), T_ind(T_ind), R_ind(R_ind), seed(seed) {
   
@@ -28,17 +29,17 @@ mgg_ssm::mgg_ssm(const Rcpp::List& model, const unsigned int seed,
 
 
 // General constructor of mgg_ssm object for approximating models
-mgg_ssm::mgg_ssm(const arma::mat& y, const arma::cube& Z, const arma::cube& H, 
-  const arma::cube& T, const arma::cube& R, const arma::vec& a1, 
-  const arma::mat& P1, const arma::cube& xreg, const arma::mat& beta, 
-  const arma::mat& D, const arma::mat& C, const unsigned int seed, 
-  const arma::uvec& Z_ind, const arma::uvec& H_ind, const arma::uvec& T_ind, 
+mgg_ssm::mgg_ssm(const arma::mat& y, const arma::cube& Z, const arma::cube& H,
+  const arma::cube& T, const arma::cube& R, const arma::vec& a1,
+  const arma::mat& P1, const arma::cube& xreg, const arma::mat& beta,
+  const arma::mat& D, const arma::mat& C, const unsigned int seed,
+  const arma::uvec& Z_ind, const arma::uvec& H_ind, const arma::uvec& T_ind,
   const arma::uvec& R_ind) :
   y(y), Z(Z), H(H), T(T), R(R), a1(a1), P1(P1), xreg(xreg), beta(beta), D(D), C(C),
   Ztv(Z.n_slices > 1), Htv(H.n_slices > 1), Ttv(T.n_slices > 1), Rtv(R.n_slices > 1),
-  Dtv(D.n_cols > 1), Ctv(C.n_cols > 1), n(y.n_cols), m(a1.n_elem), k(R.n_cols), 
-  p(y.n_rows), HH(arma::cube(p, p, Htv * (n - 1) + 1)), 
-  RR(arma::cube(m, m, Rtv * (n - 1) + 1)), 
+  Dtv(D.n_cols > 1), Ctv(C.n_cols > 1), n(y.n_cols), m(a1.n_elem), k(R.n_cols),
+  p(y.n_rows), HH(arma::cube(p, p, Htv * (n - 1) + 1)),
+  RR(arma::cube(m, m, Rtv * (n - 1) + 1)),
   xbeta(arma::mat(n, p, arma::fill::zeros)), engine(seed), zero_tol(1e-8),
   Z_ind(Z_ind), H_ind(H_ind), T_ind(T_ind), R_ind(R_ind), seed(seed) {
   
@@ -74,7 +75,7 @@ void mgg_ssm::set_theta(const arma::vec& theta) {
   }
   if(xreg.n_elem > 0) {
     for (unsigned int i = 0; i < p; i++){
-      beta.col(i) = theta.subvec(theta.n_elem - xreg.n_cols * (xreg.n_slices - i), 
+      beta.col(i) = theta.subvec(theta.n_elem - xreg.n_cols * (xreg.n_slices - i),
         theta.n_elem - xreg.n_cols * (xreg.n_slices - i + 1) - 1);
     }
     compute_xbeta();
@@ -101,7 +102,7 @@ arma::vec mgg_ssm::get_theta() const {
         R.elem(R_ind);
   }
   if(xreg.n_cols > 0) {
-    theta.subvec(theta.n_elem - xreg.n_cols * xreg.n_slices, 
+    theta.subvec(theta.n_elem - xreg.n_cols * xreg.n_slices,
       theta.n_elem - 1) = arma::vectorise(beta);
   }
   return theta;
@@ -142,38 +143,141 @@ double mgg_ssm::log_likelihood() const {
     if (obs_y.n_elem > 0) {
       arma::mat Zt = Z.slice(t * Ztv).rows(obs_y);
       arma::mat F = Zt * Pt * Zt.t() + HH.slice(t * Htv).submat(obs_y, obs_y);
-      arma::mat cholF(obs_y.n_elem, obs_y.n_elem);
-      bool chol_ok = arma::chol(cholF, F);
+      // first check to avoid armadillo warnings
+      bool chol_ok = F.is_finite();
+      if (!chol_ok) return -std::numeric_limits<double>::infinity();
+      arma::mat cholF(p, p);
+      chol_ok = arma::chol(cholF, F);
+      if (!chol_ok) return -std::numeric_limits<double>::infinity();
+      
+      arma::vec tmp = y_tmp.col(t) - D.col(t * Dtv);
+      arma::vec v = tmp.rows(obs_y) - Zt * at;
+      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+      arma::mat K = Pt * Zt.t() * inv_cholF.t() * inv_cholF;
+      at = C.col(t * Ctv) + T.slice(t * Ttv) * (at + K * v);
+      Pt = arma::symmatu(T.slice(t * Ttv) *
+        (Pt - K * F * K.t()) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
+      arma::vec Fv = inv_cholF * v;
+      logLik -= 0.5 * arma::as_scalar(obs_y.n_elem * LOG2PI +
+        2.0 * arma::accu(arma::log(arma::diagvec(cholF))) + Fv.t() * Fv);
       
-      if (chol_ok) {
-        arma::vec tmp = y_tmp.col(t) - D.col(t * Dtv);
-        arma::vec v = tmp.rows(obs_y) - Zt * at;
-        arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
-        arma::mat K = Pt * Zt.t() * inv_cholF.t() * inv_cholF;
-        at = C.col(t * Ctv) + T.slice(t * Ttv) * (at + K * v);
-        Pt = arma::symmatu(T.slice(t * Ttv) * 
-          (Pt - K * F * K.t()) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
-        arma::vec Fv = inv_cholF * v; 
-        logLik -= 0.5 * arma::as_scalar(obs_y.n_elem * LOG2PI + 
-          2.0 * arma::accu(arma::log(arma::diagvec(cholF))) + Fv.t() * Fv);
-      } else {
-        at = C.col(t * Ctv) + T.slice(t * Ttv) * at;
-        Pt = arma::symmatu(T.slice(t * Ttv) * Pt * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
-      }
     } else {
       at = C.col(t * Ctv) + T.slice(t * Ttv) * at;
       Pt = arma::symmatu(T.slice(t * Ttv) * Pt * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
     }
   }
-  
   return logLik;
 }
 
+// Kalman smoother
+void mgg_ssm::smoother(arma::mat& at, arma::cube& Pt) const {
+  
+  arma::mat y_tmp = y;
+  if(xreg.n_cols > 0) {
+    y_tmp -= xbeta.t();
+  }
+  
+  at.col(0) = a1;
+  Pt.slice(0) = P1;
+  
+  arma::mat vt(p, n, arma::fill::zeros);
+  arma::cube ZFinv(m, p, n, arma::fill::zeros);
+  arma::cube Kt(m, p, n, arma::fill::zeros);
+  
+  for (unsigned int t = 0; t < (n - 1); t++) {
+    arma::uvec na_y = arma::find_nonfinite(y_tmp.col(t));
+    if (na_y.n_elem < p) {
+      arma::mat Zt = Z.slice(t * Ztv);
+      Zt.rows(na_y).zeros();
+      arma::mat HHt = HH.slice(t * Htv);
+      HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
+      arma::mat Ft = Zt * Pt.slice(t) * Zt.t() + HHt;
+      // first check to avoid armadillo warnings
+     bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+      if (!chol_ok) {
+        at.fill(std::numeric_limits<double>::infinity()); 
+        Pt.fill(std::numeric_limits<double>::infinity());
+        return;
+      }
+      arma::mat cholF(p, p);
+      chol_ok = arma::chol(cholF, Ft);
+      if (!chol_ok) {
+        at.fill(std::numeric_limits<double>::infinity()); 
+        Pt.fill(std::numeric_limits<double>::infinity());
+        return;
+      }
+      
+      vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+      ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
+      Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
+      at.col(t + 1) = C.col(t * Ctv) +
+        T.slice(t * Ttv) * (at.col(t) + Kt.slice(t) * vt.col(t));
+      Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) *
+        (Pt.slice(t) - Kt.slice(t) * Ft * Kt.slice(t).t()) * T.slice(t * Ttv).t() +
+        RR.slice(t * Rtv));
+    } else {
+      at.col(t + 1) = C.col(t * Ctv) + T.slice(t * Ttv) * at.col(t);
+      Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) *
+        Pt.slice(t) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
+    }
+  }
+  
+  unsigned int t = n - 1;
+  arma::uvec na_y = arma::find_nonfinite(y_tmp.col(t));
+  if (na_y.n_elem < p) {
+    arma::mat Zt = Z.slice(t * Ztv);
+    Zt.rows(na_y).zeros();
+    arma::mat HHt = HH.slice(t * Htv);
+    HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
+    arma::mat Ft = Zt * Pt.slice(t) * Zt.t() + HHt;
+    // first check to avoid armadillo warnings
+   bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+    if (!chol_ok) {
+      at.fill(std::numeric_limits<double>::infinity()); 
+      Pt.fill(std::numeric_limits<double>::infinity());
+      return;
+    }
+    arma::mat cholF(p, p);
+    chol_ok = arma::chol(cholF, Ft);
+    if (!chol_ok) {
+      at.fill(std::numeric_limits<double>::infinity()); 
+      Pt.fill(std::numeric_limits<double>::infinity());
+      return;
+    }
+    vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+    arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+    ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
+    Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
+  }
+  
+  arma::vec rt(m, arma::fill::zeros);
+  arma::mat Nt(m, m, arma::fill::zeros);
+  
+  for (int t = (n - 1); t >= 0; t--) {
+    na_y = arma::find_nonfinite(y_tmp.col(t));
+    
+    if (na_y.n_elem < p) {
+      arma::mat Zt = Z.slice(t * Ztv);
+      Zt.rows(na_y).zeros();
+      arma::mat L = T.slice(t * Ttv) * (arma::eye(m, m) -
+        Kt.slice(t) * Zt);
+      rt = ZFinv.slice(t) * vt.col(t) + L.t() * rt;
+      Nt = arma::symmatu(ZFinv.slice(t) * Zt + L.t() * Nt * L);
+    } else {
+      rt = T.slice(t * Ttv).t() * rt;
+      Nt = arma::symmatu(T.slice(t * Ttv).t() * Nt * T.slice(t * Ttv));
+    }
+    at.col(t) += Pt.slice(t) * rt;
+    Pt.slice(t) -= arma::symmatu(Pt.slice(t) * Nt * Pt.slice(t));
+  }
+}
+
+
 /* Fast state smoothing, only returns smoothed estimates of states
  * which are needed in simulation smoother and Laplace approximation
  */
 arma::mat mgg_ssm::fast_smoother() const {
-  
   arma::mat y_tmp = y;
   if(xreg.n_cols > 0) {
     y_tmp -= xbeta.t();
@@ -188,7 +292,6 @@ arma::mat mgg_ssm::fast_smoother() const {
   arma::mat vt(p, n, arma::fill::zeros);
   arma::cube ZFinv(m, p, n, arma::fill::zeros);
   arma::cube Kt(m, p, n, arma::fill::zeros);
-  arma::uvec chol_ok(n, arma::fill::zeros);
   
   for (unsigned int t = 0; t < (n - 1); t++) {
     
@@ -201,26 +304,31 @@ arma::mat mgg_ssm::fast_smoother() const {
       arma::mat HHt = HH.slice(t * Htv);
       HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
       arma::mat Ft = Zt * Pt * Zt.t() + HHt;
-      
-      arma::mat cholF(p, p);
-      chol_ok(t) = arma::chol(cholF, Ft);
-      
-      if (chol_ok(t)) {
-        vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
-        arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
-        ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
-        Kt.slice(t) = Pt * ZFinv.slice(t);
-        
-        at.col(t + 1) = C.col(t * Ctv) +
-          T.slice(t * Ttv) * (at.col(t) + Kt.slice(t) * vt.col(t));
-        
-        Pt = arma::symmatu(T.slice(t * Ttv) * 
-          (Pt - Kt.slice(t) * Ft * Kt.slice(t).t()) * T.slice(t * Ttv).t() + 
-          RR.slice(t * Rtv));
-      } else {
-        at.col(t + 1) = C.col(t * Ctv) + T.slice(t * Ttv) *  at.col(t);
-        Pt = arma::symmatu(T.slice(t * Ttv) * Pt * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
+  
+    
+      bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+      if (!chol_ok) {
+        at.fill(-std::numeric_limits<double>::infinity());
+        return at;
       }
+      arma::mat cholF(p, p);
+      chol_ok = arma::chol(cholF, Ft);
+      if (!chol_ok) {
+        at.fill(-std::numeric_limits<double>::infinity());
+        return at;
+      }
+      
+      vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+      ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
+      Kt.slice(t) = Pt * ZFinv.slice(t);
+      
+      at.col(t + 1) = C.col(t * Ctv) +
+        T.slice(t * Ttv) * (at.col(t) + Kt.slice(t) * vt.col(t));
+      Pt = arma::symmatu(T.slice(t * Ttv) *
+        (Pt - Kt.slice(t) * Ft * Kt.slice(t).t()) * T.slice(t * Ttv).t() +
+        RR.slice(t * Rtv));
+     
     } else {
       at.col(t + 1) = C.col(t * Ctv) + T.slice(t * Ttv) *  at.col(t);
       Pt = arma::symmatu(T.slice(t * Ttv) * Pt * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
@@ -235,29 +343,45 @@ arma::mat mgg_ssm::fast_smoother() const {
     arma::mat HHt = HH.slice(t * Htv);
     HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
     arma::mat Ft = Zt * Pt * Zt.t() + HHt;
-    arma::mat cholF(p, p);
-    chol_ok(t) = arma::chol(cholF, Ft);
-    if (chol_ok(t)) {
-      vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
-      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
-      ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
-      Kt.slice(t) = Pt * ZFinv.slice(t);
+    bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+    if (!chol_ok) {
+      at.fill(-std::numeric_limits<double>::infinity());
+      return at;
     }
+    arma::mat cholF(p, p);
+    chol_ok = arma::chol(cholF, Ft);
+    if (!chol_ok) {
+      at.fill(-std::numeric_limits<double>::infinity());
+      return at;
+    }
+    
+    vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+    arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+    ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
+    Kt.slice(t) = Pt * ZFinv.slice(t);
+    
   }
-  
   arma::mat rt(m, n);
   rt.col(n - 1).zeros();
   for (int t = (n - 1); t > 0; t--) {
-    if (chol_ok(t)) {
-      arma::mat L = T.slice(t * Ttv) * 
-        (arma::eye(m, m) - Kt.slice(t) * Z.slice(t * Ztv));
+    na_y = arma::find_nonfinite(y_tmp.col(t));
+    
+    if (na_y.n_elem < p) {
+      arma::mat Zt = Z.slice(t * Ztv);
+      Zt.rows(na_y).zeros();
+      arma::mat L = T.slice(t * Ttv) *
+        (arma::eye(m, m) - Kt.slice(t) * Zt);
       rt.col(t - 1) = ZFinv.slice(t) * vt.col(t) + L.t() * rt.col(t);
     } else {
       rt.col(t - 1) = T.slice(t * Ttv).t() * rt.col(t);
     }
   }
-  if (chol_ok(0)){
-    arma::mat L = T.slice(0) * (arma::eye(m, m) - Kt.slice(0) * Z.slice(0));
+  na_y = arma::find_nonfinite(y_tmp.col(0));
+  
+  if (na_y.n_elem < p) {
+    arma::mat Zt = Z.slice(0);
+    Zt.rows(na_y).zeros();
+    arma::mat L = T.slice(0) * (arma::eye(m, m) - Kt.slice(0) * Zt);
     at.col(0) = a1 + P1 * (ZFinv.slice(0) * vt.col(0) + L.t() * rt.col(0));
   } else {
     at.col(0) = a1 + P1 * T.slice(0).t() * rt.col(0);
@@ -282,8 +406,6 @@ void mgg_ssm::smoother_ccov(arma::mat& at, arma::cube& Pt, arma::cube& ccov) con
   arma::mat vt(p, n, arma::fill::zeros);
   arma::cube ZFinv(m, p, n, arma::fill::zeros);
   arma::cube Kt(m, p, n, arma::fill::zeros);
-  arma::uvec chol_ok(n, arma::fill::zeros);
-  
   
   for (unsigned int t = 0; t < (n - 1); t++) {
     arma::uvec na_y = arma::find_nonfinite(y_tmp.col(t));
@@ -293,27 +415,37 @@ void mgg_ssm::smoother_ccov(arma::mat& at, arma::cube& Pt, arma::cube& ccov) con
       arma::mat HHt = HH.slice(t * Htv);
       HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
       arma::mat Ft = Zt * Pt.slice(t) * Zt.t() + HHt;
-      arma::mat cholF(p, p);
-      chol_ok(t) = arma::chol(cholF, Ft);
-      
-      if (chol_ok(t)) {
-        vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
-        arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
-        ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
-        Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
-        at.col(t + 1) = C.col(t * Ctv) + 
-          T.slice(t * Ttv) * (at.col(t) + Kt.slice(t) * vt.col(t));
-        Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) * 
-          (Pt.slice(t) - Kt.slice(t) * Ft * Kt.slice(t).t()) * T.slice(t * Ttv).t() + 
-          RR.slice(t * Rtv));
-      } else {
-        at.col(t + 1) = C.col(t * Ctv) + T.slice(t * Ttv) *  at.col(t);
-        Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) * 
-          Pt.slice(t) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
+      // first check to avoid armadillo warnings
+      bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+      if (!chol_ok) {
+        at.fill(std::numeric_limits<double>::infinity()); 
+        Pt.fill(std::numeric_limits<double>::infinity());
+        ccov.fill(std::numeric_limits<double>::infinity());
+        return;
       }
+      arma::mat cholF(p, p);
+      chol_ok = arma::chol(cholF ,Ft);
+      if (!chol_ok) {
+        at.fill(std::numeric_limits<double>::infinity()); 
+        Pt.fill(std::numeric_limits<double>::infinity());
+        ccov.fill(std::numeric_limits<double>::infinity());
+        return;
+      }
+      
+      
+      vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+      ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
+      Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
+      at.col(t + 1) = C.col(t * Ctv) +
+        T.slice(t * Ttv) * (at.col(t) + Kt.slice(t) * vt.col(t));
+      Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) *
+        (Pt.slice(t) - Kt.slice(t) * Ft * Kt.slice(t).t()) * T.slice(t * Ttv).t() +
+        RR.slice(t * Rtv));
+      
     } else {
       at.col(t + 1) = C.col(t * Ctv) + T.slice(t * Ttv) * at.col(t);
-      Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) * 
+      Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) *
         Pt.slice(t) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
     }
     ccov.slice(t) = Pt.slice(t + 1); //store for smoothing;
@@ -327,19 +459,29 @@ void mgg_ssm::smoother_ccov(arma::mat& at, arma::cube& Pt, arma::cube& ccov) con
     arma::mat HHt = HH.slice(t * Htv);
     HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
     arma::mat Ft = Zt * Pt.slice(t) * Zt.t() + HHt;
-    arma::mat cholF(p, p);
-    chol_ok(t) = arma::chol(cholF, Ft);
-    if (chol_ok(t)) {
-      vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
-      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
-      ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
-      Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
-      ccov.slice(t) = arma::symmatu(T.slice(t * Ttv) * (Pt.slice(t) -
-        Kt.slice(t) * Ft * Kt.slice(t).t()) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
-    } else {
-      ccov.slice(t) = arma::symmatu(T.slice(t * Ttv) * 
-        Pt.slice(t) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
+    // first check to avoid armadillo warnings
+   bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+    if (!chol_ok) {
+      at.fill(std::numeric_limits<double>::infinity()); 
+      Pt.fill(std::numeric_limits<double>::infinity());
+      ccov.fill(std::numeric_limits<double>::infinity());
+      return;
     }
+    arma::mat cholF(p, p);
+    chol_ok = arma::chol(cholF, Ft);
+    if (!chol_ok) {
+      at.fill(std::numeric_limits<double>::infinity()); 
+      Pt.fill(std::numeric_limits<double>::infinity());
+      ccov.fill(std::numeric_limits<double>::infinity());
+      return;
+    }
+    vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+    arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+    ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
+    Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
+    ccov.slice(t) = arma::symmatu(T.slice(t * Ttv) * (Pt.slice(t) -
+      Kt.slice(t) * Ft * Kt.slice(t).t()) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
+    
     
   } else {
     ccov.slice(t) = arma::symmatu(T.slice(t * Ttv) * Pt.slice(t) * T.slice(t * Ttv).t() +
@@ -349,16 +491,19 @@ void mgg_ssm::smoother_ccov(arma::mat& at, arma::cube& Pt, arma::cube& ccov) con
   arma::vec rt(m, arma::fill::zeros);
   arma::mat Nt(m, m, arma::fill::zeros);
   
-  for (int t = (n - 1); t >= 0; t--) {
-    if (chol_ok(t)){
-      arma::mat L = T.slice(t * Ttv) * (arma::eye(m, m) - 
-        Kt.slice(t) * Z.slice(t * Ztv));
+  for (int t = (n - 1); t >= 0; t--) {    
+    na_y = arma::find_nonfinite(y_tmp.col(t));
+    if (na_y.n_elem < p) {
+      arma::mat Zt = Z.slice(t * Ztv);
+      Zt.rows(na_y).zeros();
+      arma::mat L = T.slice(t * Ttv) * (arma::eye(m, m) -
+        Kt.slice(t) * Zt);
       //P[t+1] stored to ccov_t
       ccov.slice(t) = Pt.slice(t) * L.t() * (arma::eye(m, m) - Nt * ccov.slice(t));
       rt = ZFinv.slice(t) * vt.col(t) + L.t() * rt;
-      Nt = arma::symmatu(ZFinv.slice(t) * Z.slice(t * Ztv) + L.t() * Nt * L);
+      Nt = arma::symmatu(ZFinv.slice(t) * Zt + L.t() * Nt * L);
     } else {
-      ccov.slice(t) = Pt.slice(t) * T.slice(t * Ttv).t() * 
+      ccov.slice(t) = Pt.slice(t) * T.slice(t * Ttv).t() *
         (arma::eye(m, m) - Nt * ccov.slice(t));
       rt = T.slice(t * Ttv).t() * rt;
       Nt = arma::symmatu(T.slice(t * Ttv).t() * Nt * T.slice(t * Ttv));
@@ -370,7 +515,7 @@ void mgg_ssm::smoother_ccov(arma::mat& at, arma::cube& Pt, arma::cube& ccov) con
 }
 
 
-void mgg_ssm::filter(arma::mat& at, arma::mat& att, 
+double mgg_ssm::filter(arma::mat& at, arma::mat& att,
   arma::cube& Pt, arma::cube& Ptt) const {
   
   arma::mat y_tmp = y;
@@ -381,47 +526,55 @@ void mgg_ssm::filter(arma::mat& at, arma::mat& att,
   at.col(0) = a1;
   Pt.slice(0) = P1;
   
-  arma::mat vt(p, n, arma::fill::zeros);
-  arma::cube ZFinv(m, p, n, arma::fill::zeros);
-  arma::cube Kt(m, p, n, arma::fill::zeros);
-  arma::uvec chol_ok(n, arma::fill::zeros);
-  
+  const double LOG2PI = std::log(2.0 * M_PI);
+  double logLik = 0.0;
   
   for (unsigned int t = 0; t < (n - 1); t++) {
     arma::uvec na_y = arma::find_nonfinite(y_tmp.col(t));
+    
     if (na_y.n_elem < p) {
       arma::mat Zt = Z.slice(t * Ztv);
       Zt.rows(na_y).zeros();
       arma::mat HHt = HH.slice(t * Htv);
       HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
       arma::mat Ft = Zt * Pt.slice(t) * Zt.t() + HHt;
-      arma::mat cholF(p, p);
-      chol_ok(t) = arma::chol(cholF, Ft);
-      
-      if (chol_ok(t)) {
-        vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
-        arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
-        ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
-        Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
-        att.col(t) = at.col(t) + Kt.slice(t) * vt.col(t);
-        at.col(t + 1) = C.col(t * Ctv) + 
-          T.slice(t * Ttv) * att.col(t);
-        Ptt.slice(t) = Pt.slice(t) - Kt.slice(t) * Ft * Kt.slice(t).t();
-        Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) * 
-          Ptt.slice(t) * T.slice(t * Ttv).t() + 
-          RR.slice(t * Rtv));
-      } else {
-        att.col(t) = at.col(t);
-        at.col(t + 1) = C.col(t * Ctv) + T.slice(t * Ttv) *  att.col(t);
-        Ptt.slice(t) = Pt.slice(t);
-        Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) * 
-          Ptt.slice(t) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
+     bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+      if (!chol_ok) {
+        at.fill(std::numeric_limits<double>::infinity()); 
+        Pt.fill(std::numeric_limits<double>::infinity());
+        att.fill(std::numeric_limits<double>::infinity());
+        Ptt.fill(std::numeric_limits<double>::infinity());
+        return -std::numeric_limits<double>::infinity();
       }
+      arma::mat cholF(p, p);
+      chol_ok = arma::chol(cholF ,Ft);
+      if (!chol_ok) {
+        at.fill(std::numeric_limits<double>::infinity()); 
+        at.fill(std::numeric_limits<double>::infinity()); 
+        Pt.fill(std::numeric_limits<double>::infinity());
+        att.fill(std::numeric_limits<double>::infinity());
+        Ptt.fill(std::numeric_limits<double>::infinity());
+        return -std::numeric_limits<double>::infinity();
+      }
+        arma::vec v = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+        arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+        arma::mat ZFinv = Zt.t() * inv_cholF.t() * inv_cholF;
+        arma::mat K = Pt.slice(t) * ZFinv;
+        att.col(t) = at.col(t) + K * v;
+        at.col(t + 1) = C.col(t * Ctv) +
+          T.slice(t * Ttv) * att.col(t);
+        Ptt.slice(t) = Pt.slice(t) - K * Ft * K.t();
+        Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) *
+          Ptt.slice(t) * T.slice(t * Ttv).t() +
+          RR.slice(t * Rtv));
+        arma::vec Fv = inv_cholF * v;
+        logLik -= 0.5 * arma::as_scalar((p - na_y.n_elem) * LOG2PI +
+          2.0 * arma::accu(arma::log(arma::diagvec(cholF))) + Fv.t() * Fv);
     } else {
       att.col(t) = at.col(t);
       at.col(t + 1) = C.col(t * Ctv) + T.slice(t * Ttv) *  att.col(t);
       Ptt.slice(t) = Pt.slice(t);
-      Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) * 
+      Pt.slice(t + 1) = arma::symmatu(T.slice(t * Ttv) *
         Ptt.slice(t) * T.slice(t * Ttv).t() + RR.slice(t * Rtv));
     }
   }
@@ -434,35 +587,45 @@ void mgg_ssm::filter(arma::mat& at, arma::mat& att,
     arma::mat HHt = HH.slice(t * Htv);
     HHt.submat(na_y, na_y) = arma::eye(na_y.n_elem, na_y.n_elem);
     arma::mat Ft = Zt * Pt.slice(t) * Zt.t() + HHt;
-    arma::mat cholF(p, p);
-    chol_ok(t) = arma::chol(cholF, Ft);
-    if (chol_ok(t)) {
-      vt.col(t) = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
-      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
-      ZFinv.slice(t) = Zt.t() * inv_cholF.t() * inv_cholF;
-      Kt.slice(t) = Pt.slice(t) * ZFinv.slice(t);
-      att.col(t) = at.col(t) + Kt.slice(t) * vt.col(t);
-      Ptt.slice(t) = Pt.slice(t) - Kt.slice(t) * Ft * Kt.slice(t).t();
-    } else {
-      att.col(t) = at.col(t);
-      Ptt.slice(t) = Pt.slice(t);
+   bool chol_ok = Ft.is_finite() && arma::all(Ft.diag() > 0);
+    if (!chol_ok) {
+      at.fill(std::numeric_limits<double>::infinity()); 
+      Pt.fill(std::numeric_limits<double>::infinity());
+      att.fill(std::numeric_limits<double>::infinity());
+      Ptt.fill(std::numeric_limits<double>::infinity());
+      return -std::numeric_limits<double>::infinity();
     }
+    arma::mat cholF(p, p);
+    chol_ok = arma::chol(cholF ,Ft);
+    if (!chol_ok) {
+      at.fill(std::numeric_limits<double>::infinity()); 
+      at.fill(std::numeric_limits<double>::infinity()); 
+      Pt.fill(std::numeric_limits<double>::infinity());
+      att.fill(std::numeric_limits<double>::infinity());
+      Ptt.fill(std::numeric_limits<double>::infinity());
+      return -std::numeric_limits<double>::infinity();
+    }
+      arma::vec v = y_tmp.col(t) - D.col(t * Dtv) - Zt * at.col(t);
+      arma::mat inv_cholF = arma::inv(arma::trimatu(cholF));
+      arma::mat ZFinv = Zt.t() * inv_cholF.t() * inv_cholF;
+      arma::mat K = Pt.slice(t) * ZFinv;
+      att.col(t) = at.col(t) + K * v;
+      Ptt.slice(t) = Pt.slice(t) - K * Ft * K.t();
+      arma::vec Fv = inv_cholF * v;
+      logLik -= 0.5 * arma::as_scalar((p - na_y.n_elem) * LOG2PI +
+        2.0 * arma::accu(arma::log(arma::diagvec(cholF))) + Fv.t() * Fv);
+   
   } else {
     att.col(t) = at.col(t);
     Ptt.slice(t) = Pt.slice(t);
   }
+  return logLik;
 }
 
 
 arma::cube mgg_ssm::simulate_states() {
   
-  arma::uvec nonzero = arma::find(P1.diag() > 0);
-  arma::mat L_P1(m, m, arma::fill::zeros);
-  if (nonzero.n_elem > 0) {
-    L_P1.submat(nonzero, nonzero) =
-      arma::chol(P1.submat(nonzero, nonzero), "lower");
-  }
-  
+  arma::mat L_P1 = psd_chol(P1);
   arma::cube asim(m, n, 1);
   
   std::normal_distribution<> normal(0.0, 1.0);
@@ -472,7 +635,7 @@ arma::cube mgg_ssm::simulate_states() {
     um(j) = normal(engine);
   }
   asim.slice(0).col(0) = L_P1 * um;
-  arma::vec y_tmp = y;
+  arma::mat y_tmp = y;
   for (unsigned int t = 0; t < (n - 1); t++) {
     arma::uvec na_y = arma::find_nonfinite(y.col(t));
     if (na_y.n_elem < p) {
