@@ -6,16 +6,30 @@
 #include <sitmo.h>
 #include "bssm.h"
 #include "mgg_ssm.h"
-#include "function_pointers.h"
+
+
+// typedef for a pointer of nonlinear function of model equation returning vec (T, Z)
+typedef arma::vec (*nvec_fnPtr)(const unsigned int t, const arma::vec& alpha, const arma::vec& theta, 
+  const arma::vec& known_params, const arma::mat& known_tv_params);
+// typedef for a pointer of nonlinear function of model equation returning mat (Tg, Zg, H, R)
+typedef arma::mat (*nmat_fnPtr)(const unsigned int t, const arma::vec& alpha, const arma::vec& theta, 
+  const arma::vec& known_params, const arma::mat& known_tv_params);
+
+// typedef for a pointer returning a1
+typedef arma::vec (*a1_fnPtr)(const arma::vec& theta, const arma::vec& known_params);
+// typedef for a pointer returning P1
+typedef arma::mat (*P1_fnPtr)(const arma::vec& theta, const arma::vec& known_params);
+// typedef for a pointer of log-prior function
+typedef double (*prior_fnPtr)(const arma::vec&);
 
 
 class nlg_ssm {
   
 public:
   
-  nlg_ssm(const arma::mat& y, SEXP Z_fn_, SEXP H_fn_, SEXP T_fn_, SEXP R_fn_, 
-    SEXP Z_gn_, SEXP T_gn_, SEXP a1_fn_, SEXP P1_fn_,
-    const arma::vec& theta, SEXP log_prior_pdf_, const arma::vec& known_params, 
+  nlg_ssm(const arma::mat& y, nvec_fnPtr Z_fn_, nmat_fnPtr H_fn_, nvec_fnPtr T_fn_, 
+    nmat_fnPtr R_fn_, nmat_fnPtr Z_gn_, nmat_fnPtr T_gn_, a1_fnPtr a1_fn_, P1_fnPtr P1_fn_,
+    const arma::vec& theta, prior_fnPtr log_prior_pdf_, const arma::vec& known_params, 
     const arma::mat& known_tv_params, const unsigned int m, const unsigned int k,
     const arma::uvec& time_varying, const unsigned int seed);
   
@@ -27,10 +41,16 @@ public:
   arma::mat approximate(mgg_ssm& approx_model, const unsigned int max_iter, 
     const double conv_tol) const;
   
+  Rcpp::List predict_interval(const arma::vec& probs, const arma::mat& thetasim,
+    const arma::mat& alpha_last, const arma::cube& P_last, 
+    const arma::uvec& counts, const unsigned int predict_type);
   
   arma::cube predict_sample(const arma::mat& thetasim, const arma::mat& alpha, 
-    const arma::uvec& counts, const unsigned int predict_type);
-  arma::mat sample_model(const arma::vec& a1_sim, const unsigned int predict_type);
+    const arma::uvec& counts, const unsigned int predict_type, 
+    const unsigned int nsim);
+  
+  arma::cube sample_model(const arma::vec& a1_sim, 
+    const unsigned int predict_type, const unsigned int nsim);
   
   double ekf(arma::mat& at, arma::mat& att, arma::cube& Pt, 
     arma::cube& Ptt, const unsigned int iekf_iter) const;
@@ -44,11 +64,11 @@ public:
     const double alpha = 1.0, const double beta = 0.0, const double kappa = 2.0) const;
   
     // bootstrap filter  
-  double bsf_filter(const unsigned int nsim, arma::cube& alphasim, 
+  double bsf_filter(const unsigned int nsim, arma::cube& alpha, 
     arma::mat& weights, arma::umat& indices);
   
   // auxiliary particle filter  
-  double aux_filter(const unsigned int nsim, arma::cube& alphasim, 
+  double aux_filter(const unsigned int nsim, arma::cube& alpha, 
     arma::mat& weights, arma::umat& indices);
   
   // psi-particle filter
@@ -60,26 +80,18 @@ public:
   double ekf_filter(const unsigned int nsim, arma::cube& alpha,
     arma::mat& weights, arma::umat& indices);
   
-  // defensive psi-particle filter
-  double df_psi_filter(const mgg_ssm& approx_model, const double approx_loglik,
-    const unsigned int nsim, arma::cube& alpha, arma::mat& weights,
-    arma::umat& indices);
-  
   // compute logarithms of _unnormalized_ importance weights g(y_t | alpha_t) / ~g(~y_t | alpha_t)
   arma::vec log_weights(const mgg_ssm& approx_model, 
-    const unsigned int t, const arma::cube& alphasim, const arma::mat& alpha_prev) const;
-  // compute logarithms of _unnormalized_ importance weights g(y_t | alpha_t) / ~g(~y_t | alpha_t)
-  arma::vec log_weights_df(const mgg_ssm& approx_model, 
-    const unsigned int t, const arma::cube& alphasim, const arma::mat& alpha_prev) const;
-  
+    const unsigned int t, const arma::cube& alpha, const arma::mat& alpha_prev) const;
+
   // compute unnormalized mode-based scaling terms
   // log[g(y_t | ^alpha_t) / ~g(y_t | ^alpha_t)]
   arma::vec scaling_factors(const mgg_ssm& approx_model, const arma::mat& mode_estimate) const;
   
   // compute logarithms of _unnormalized_ densities g(y_t | alpha_t)
-  arma::vec log_obs_density(const unsigned int t, const arma::cube& alphasim) const;
+  arma::vec log_obs_density(const unsigned int t, const arma::cube& alpha) const;
   // compute logarithms of _unnormalized_ densities g(y_t | alpha_t)
-  double log_obs_density(const unsigned int t, const arma::vec& alphasim) const;
+  double log_obs_density(const unsigned int t, const arma::vec& alpha) const;
   
   void ekf_update_step(const unsigned int t, const arma::vec y, 
     const arma::vec& at, const arma::mat& Pt, arma::vec& att, arma::mat& Ptt) const;
@@ -91,21 +103,21 @@ public:
   // y_t = Z(alpha_t, theta,t) + H(theta,t)*eps_t, 
   // alpha_t+1 = T(alpha_t, theta,t) + R(theta, t)*eta_t
   
-  vec_fn Z_fn;
-  mat_varfn H_fn;
-  vec_fn T_fn;
-  mat_varfn R_fn;
+  nvec_fnPtr Z_fn;
+  nmat_fnPtr H_fn;
+  nvec_fnPtr T_fn;
+  nmat_fnPtr R_fn;
   //and the derivatives
-  mat_fn Z_gn;
-  mat_fn T_gn;
+  nmat_fnPtr Z_gn;
+  nmat_fnPtr T_gn;
   //initial value
-  vec_initfn a1_fn;
-  mat_initfn P1_fn;
+  a1_fnPtr a1_fn;
+  P1_fnPtr P1_fn;
   
   // Parameter vector used in _all_ nonlinear functions
   arma::vec theta;
   //prior log-pdf
-  double_fn log_prior_pdf;
+  prior_fnPtr log_prior_pdf;
   // vector of known parameters
   arma::vec known_params;
   // matrix of known (time-varying) parameters
